from django import forms

from .utils import render_commonmark, CommonMarkMedia
from .widgets import CommonMarkTextarea

from wagtail.core.blocks import TextBlock


class CommonMarkBlock(CommonMarkMedia, TextBlock):
    def __init__(self, required=True, help_text=None, **kwargs):
        self.field = forms.CharField(required=required, help_text=help_text,
                                     widget=CommonMarkTextarea())
        super().__init__(**kwargs)

    def render_basic(self, value, context=None):
        return render_commonmark(value, context)



from django import forms
from django.utils.encoding import smart_text
from django.utils.safestring import mark_safe

import bleach
import commonmark


MARKDOWN_TAGS = [
    'h1', 'h2', 'h3', 'h4', 'h5', 'h6',
    'b', 'i', 'strong', 'em', 'tt',
    'p', 'br',
    'span', 'div', 'blockquote', 'pre', 'code', 'hr',
    'ul', 'ol', 'li', 'dd', 'dt',
    'img',
    'a',
    'sub', 'sup',
]
MARKDOWN_ATTRS = {
    '*': ['id'],
    'img': ['src', 'alt', 'title'],
    'a': ['href', 'alt', 'title'],
}


def render_commonmark(text, context=None):
    """
    Turn commonmark into HTML.
    """
    if context is None or not isinstance(context, dict):
        context = {}
    commonmark_html = _transform_commonmark_into_html(text)
    sanitised_commonmark_html = _sanitise_commonmark_html(commonmark_html)
    return mark_safe(sanitised_commonmark_html)


def _transform_commonmark_into_html(text):
    return commonmark.commonmark(smart_text(text))


def _sanitise_commonmark_html(commonmark_html):
    return bleach.clean(
        commonmark_html,
        tags=MARKDOWN_TAGS,
        attributes=MARKDOWN_ATTRS)


class CommonMarkMedia:
    """
    Mixin providing the common JS and CSS for the CommonMark admin panels
    """
    @property
    def media(self):
        return forms.Media(
            css={
                'all': (
                    'wagtailcommonmark/css/codemirror.custom.css',
                    'wagtailcommonmark/css/codemirror.css',
                )
            },
            js=(
                'wagtailcommonmark/js/codemirror.js',
                'wagtailcommonmark/js/codemirror.addon.edit.continuelist.js',
                'wagtailcommonmark/js/codemirror.mode.markdown.js',
                'wagtailcommonmark/js/codemirror.attach.js',
            )
        )



from django.db.models import TextField

from .widgets import CommonMarkTextarea


class CommonMarkField(TextField):
    def formfield(self, **kwargs):
        defaults = {
            'widget': CommonMarkTextarea
        }
        defaults.update(kwargs)
        return super().formfield(**defaults)



from django import forms

from wagtail.utils.widgets import WidgetWithScript

from .utils import CommonMarkMedia


class CommonMarkTextarea(CommonMarkMedia,
                         WidgetWithScript,
                         forms.widgets.Textarea):
    def render_js_init(self, id_, name, value):
        return 'codeMirrorAttach("{0}");'.format(id_)



from setuptools import setup, find_packages
import subprocess


def get_git_revision_hash():
    try:
        git_hash = subprocess.check_output(['git', 'rev-parse', 'HEAD'])
    except subprocess.CalledProcessError:
        return 'master'
    else:
        return git_hash.decode('ascii').splitlines()[0]


README = 'https://github.com/belak/wagtail-commonmark/blob/{hash}/README.md'
README = README.format(
    hash=get_git_revision_hash()
)


INSTALL_REQUIRES = [
    'bleach>=1.4.2,<2.2',
    'Wagtail>=2.0',
]


CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Framework :: Wagtail',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python',
]


setup(
    name='wagtail-commonmark',
    version='0.1',
    description='Commonmark support for Wagtail',
    long_description="Provides CommonMark page field and streamfield block for "
                     "Wagtail. More info: {}".format(README),
    author='Kaleb Elwert',
    author_email='belak@coded.io',
    url='https://github.com/torchbox/wagtail-markdown',
    install_requires=INSTALL_REQUIRES,
    license='zlib',
    packages=find_packages(),
    include_package_data=True,
    classifiers=CLASSIFIERS,
)
